@@ -7,478 +7,18 @@ from sklearn.decomposition import PCA
 from sklearn.covariance import LedoitWolf
 import warnings
 from collections import defaultdict, deque
-from dataclasses import dataclass
-from typing import Dict, List, Tuple, Optional
 import logging
 from scipy.optimize import minimize
-
+from Backtester_Tushar.Risk.risk_limits import RiskLimits
+from Backtester_Tushar.Portfolio_Management_and_Tracker.utils import *
+from Backtester_Tushar.Strategy.signal_processing import SignalProcessor
+from Backtester_Tushar.Execution.t_cost import MarketImpactModel
+from Backtester_Tushar.Risk.risk_manager import RiskManager
+from Backtester_Tushar.Portfolio_Management_and_Tracker.analyzer import PerformanceAnalyzer
 warnings.filterwarnings('ignore')
 
 
-@dataclass
-class RiskLimits:
-    """Risk limit configuration"""
-    max_position_size: float = 0.05  # 5% max per position
-    max_sector_exposure: float = 0.20  # 20% max per sector
-    max_beta_exposure: float = 0.15  # Net beta limit
-    max_daily_var: float = 0.02  # 2% daily VaR
-    max_drawdown: float = 0.10  # 10% max drawdown
-    max_leverage: float = 1.0  # Net leverage limit
-
-
-@njit
-def fast_softmax(x):
-    """Numerically stable softmax with numba acceleration"""
-    x_shifted = x - np.max(x)
-    exp_x = np.exp(x_shifted)
-    return exp_x / np.sum(exp_x)
-
-
-@njit
-def fast_sharpe_ratio(returns, risk_free_rate=0.0):
-    """Fast Sharpe ratio calculation"""
-    excess_returns = returns - risk_free_rate
-    if np.std(excess_returns) == 0:
-        return 0.0
-    return np.mean(excess_returns) / np.std(excess_returns)
-
-
-@njit
-def exponential_decay(values, half_life):
-    """Apply exponential decay to signal values"""
-    decay_factor = np.log(0.5) / half_life
-    n = len(values)
-    weights = np.exp(decay_factor * np.arange(n - 1, -1, -1))
-    return values * weights
-
-
-class SignalProcessor:
-    """Advanced signal processing with decay, orthogonalization, and quality metrics"""
-
-    def __init__(self, half_life_days=5, min_ic_threshold=0.02):
-        self.half_life_days = half_life_days
-        self.min_ic_threshold = min_ic_threshold
-        self.signal_history = defaultdict(deque)
-        self.ic_history = defaultdict(deque)
-
-    def decay_signals(self, signals_df):
-        """Apply exponential decay to signals based on age"""
-        decayed_signals = signals_df.copy()
-
-        for ticker in signals_df['ticker'].unique():
-            ticker_data = signals_df[signals_df['ticker'] == ticker].sort_values('date')
-            if len(ticker_data) > 1:
-                signal_values = ticker_data['signal_descriptor'].values
-                decayed_values = exponential_decay(signal_values, self.half_life_days)
-                decayed_signals.loc[decayed_signals['ticker'] == ticker, 'signal_descriptor'] = decayed_values
-
-        return decayed_signals
-
-    def calculate_information_coefficient(self, signals, forward_returns, periods=20):
-        """Calculate rolling information coefficient"""
-        if len(signals) != len(forward_returns) or len(signals) < periods:
-            return 0.0
-
-        ic_values = []
-        for i in range(periods, len(signals)):
-            window_signals = signals[i - periods:i]
-            window_returns = forward_returns[i - periods:i]
-
-            if len(window_signals) > 5:  # Minimum sample size
-                ic, _ = stats.spearmanr(window_signals, window_returns)
-                if not np.isnan(ic):
-                    ic_values.append(ic)
-
-        return np.mean(ic_values) if ic_values else 0.0
-
-    def orthogonalize_signals(self, signal_matrix):
-        """Orthogonalize signals using Gram-Schmidt process"""
-        if signal_matrix.shape[1] < 2:
-            return signal_matrix
-
-        # Handle NaN values
-        signal_matrix = np.nan_to_num(signal_matrix, nan=0.0)
-
-        # Apply Gram-Schmidt orthogonalization
-        orthogonal_signals = np.zeros_like(signal_matrix)
-
-        for i in range(signal_matrix.shape[1]):
-            vector = signal_matrix[:, i].copy()
-
-            for j in range(i):
-                projection = np.dot(vector, orthogonal_signals[:, j]) / np.dot(orthogonal_signals[:, j],
-                                                                               orthogonal_signals[:, j])
-                if not np.isnan(projection):
-                    vector -= projection * orthogonal_signals[:, j]
-
-            norm = np.linalg.norm(vector)
-            orthogonal_signals[:, i] = vector / norm if norm > 1e-10 else vector
-
-        return orthogonal_signals
-
-
-class MarketImpactModel:
-    """Almgren-Chriss execution model with market impact"""
-
-    def __init__(self, permanent_impact=0.1, temporary_impact=0.01, volatility_adjustment=True):
-        self.permanent_impact = permanent_impact
-        self.temporary_impact = temporary_impact
-        self.volatility_adjustment = volatility_adjustment
-
-    def calculate_market_impact(self, shares, avg_volume, volatility, participation_rate=0.1):
-        """Calculate market impact cost using Almgren-Chriss model"""
-        if avg_volume <= 0 or shares == 0:
-            return 0.0
-
-        # Market impact parameters
-        volume_ratio = abs(shares) / avg_volume
-
-        # Permanent impact (linear in volume)
-        permanent_cost = self.permanent_impact * volume_ratio
-
-        # Temporary impact (square-root law)
-        temporary_cost = self.temporary_impact * np.sqrt(volume_ratio)
-
-        # Volatility adjustment
-        if self.volatility_adjustment and volatility > 0:
-            vol_multiplier = volatility / 0.02  # Normalize to 2% daily vol
-            permanent_cost *= vol_multiplier
-            temporary_cost *= vol_multiplier
-
-        total_impact = permanent_cost + temporary_cost
-        return min(total_impact, 0.05)  # Cap at 5%
-
-    def optimal_execution_schedule(self, total_shares, time_horizon_days, risk_aversion=1e-6):
-        """Calculate optimal execution schedule"""
-        if time_horizon_days <= 0:
-            return [total_shares]
-
-        # Simple arithmetic schedule for now (can be enhanced with more complex optimization)
-        shares_per_period = total_shares / time_horizon_days
-        return [shares_per_period] * int(time_horizon_days)
-
-
-class RiskManager:
-    """Comprehensive risk management with VaR, factor exposure, and compliance"""
-
-    def __init__(self, risk_limits: RiskLimits, lookback_days=252):
-        self.risk_limits = risk_limits
-        self.lookback_days = lookback_days
-        self.factor_loadings = {}
-        self.correlation_matrix = None
-        self.var_models = {}
-
-    def calculate_portfolio_var(self, positions, returns_covariance, confidence=0.05):
-        """Calculate portfolio VaR as % of portfolio value (not absolute dollars)."""
-        if not positions or returns_covariance is None:
-            return 0.0
-
-        tickers = list(positions.keys())
-        position_values = np.array([positions[ticker]['market_value'] for ticker in tickers])
-
-        if len(position_values) == 0:
-            return 0.0
-
-        portfolio_value = np.sum(np.abs(position_values))
-        if portfolio_value == 0:
-            return 0.0
-
-        weights = position_values / portfolio_value
-
-        n_assets = len(weights)
-        if returns_covariance.shape[0] != n_assets:
-            return 0.0
-
-        # Portfolio variance (in return space)
-        portfolio_variance = np.dot(weights.T, np.dot(returns_covariance, weights))
-        if portfolio_variance < 0:
-            portfolio_variance = 0
-        portfolio_std = np.sqrt(portfolio_variance)
-
-        # Parametric VaR (as % of portfolio value)
-        z_score = norm.ppf(1 - confidence)  # e.g. 1.645 for 95%
-        parametric_var_pct = z_score * portfolio_std  # % loss
-
-        # Historical VaR fallback (also in % terms)
-        if hasattr(self, 'historical_returns') and len(self.historical_returns) > 100:
-            hist_var_pct = -np.percentile(self.historical_returns, confidence * 100)
-            return max(parametric_var_pct, hist_var_pct)
-
-        return parametric_var_pct
-
-    def optimize_with_constraints(self, expected_returns, covariance_matrix, sector_groups, betas=None,
-                                  risk_aversion=1.0, is_short=False):
-        """
-        Mean-variance optimization with position, sector, and beta constraints.
-        Args:
-            expected_returns (np.array): Expected returns for each asset.
-            covariance_matrix (np.array): Covariance matrix of returns.
-            sector_groups (dict): Mapping of sector names to indices of assets in that sector.
-            betas (np.array, optional): Beta values for each asset, default None.
-            risk_aversion (float): Risk aversion factor, default 1.0.
-            is_short (bool): Whether optimizing for short positions, default False.
-        Returns:
-            np.array: Optimized weights.
-        """
-        n_assets = len(expected_returns)
-        if n_assets == 0 or covariance_matrix is None or covariance_matrix.shape[0] != n_assets:
-            return np.zeros(n_assets)
-
-        # Validate inputs
-        if np.any(np.isnan(expected_returns)) or np.any(np.isnan(covariance_matrix)):
-            return np.zeros(n_assets)
-        expected_returns = np.nan_to_num(expected_returns, nan=0.0)
-        covariance_matrix = np.nan_to_num(covariance_matrix, nan=0.0)
-
-        # Regularize covariance matrix to avoid ill-conditioning
-        regularization = 1e-6 * np.eye(n_assets)
-        cov_reg = covariance_matrix + regularization
-
-        # Objective function: minimize risk - (1/risk_aversion) * return
-        def objective(weights):
-            if risk_aversion <= 0:
-                return np.inf  # Prevent division by zero or negative risk aversion
-            port_var = np.dot(weights.T, np.dot(cov_reg, weights))
-            port_ret = np.dot(weights, expected_returns)
-            return port_var - (1 / risk_aversion) * port_ret
-
-        # Helper function to create position size constraint
-        def create_position_constraint(index):
-            def constraint(weights):
-                return self.risk_limits.max_position_size - abs(weights[index])
-
-            return constraint
-
-        # Helper function to create sector exposure constraint
-        def create_sector_constraint(indices):
-            def constraint(weights):
-                return self.risk_limits.max_sector_exposure - np.sum(np.abs(weights[indices]))
-
-            return constraint
-
-        # Helper function to create beta constraint
-        def create_beta_constraint():
-            def constraint(weights):
-                return self.risk_limits.max_beta_exposure - abs(np.dot(weights, betas))
-
-            return constraint
-
-        # Build constraints list
-        constraints = [
-            {'type': 'eq', 'fun': lambda w: np.sum(w) - 1}  # Weights sum to 1
-        ]
-
-        # Add position size constraints
-        for i in range(n_assets):
-            constraints.append({'type': 'ineq', 'fun': create_position_constraint(i)})
-
-        # Add sector exposure constraints
-        for sector, indices in sector_groups.items():
-            if not all(0 <= idx < n_assets for idx in indices):  # Validate indices
-                continue
-            constraints.append({'type': 'ineq', 'fun': create_sector_constraint(indices)})
-
-        # Add beta constraint if betas provided
-        if betas is not None and len(betas) == n_assets:
-            constraints.append({'type': 'ineq', 'fun': create_beta_constraint()})
-
-        # Bounds: 0 to 1 for longs, -1 to 0 for shorts
-        bounds = [(0, 1) if not is_short else (-1, 0) for _ in range(n_assets)]
-
-        # Initial guess
-        init_weights = np.ones(n_assets) / n_assets
-        if is_short:
-            init_weights = -init_weights  # Start with short bias
-
-        # Optimization
-        result = minimize(objective, init_weights, method='SLSQP', bounds=bounds, constraints=constraints,
-                          options={'maxiter': 1000, 'ftol': 1e-8})
-
-        if result.success:
-            weights = result.x
-        else:
-            # Fallback: equal weights with iterative constraint adjustment
-            weights = np.ones(n_assets) / n_assets
-            if is_short:
-                weights = -weights
-            max_iterations = 20
-            for _ in range(max_iterations):
-                violations = False
-                # Check position size
-                pos_viol = np.max(np.abs(weights) - self.risk_limits.max_position_size)
-                if pos_viol > 0:
-                    violations = True
-                    weights[np.abs(weights) > self.risk_limits.max_position_size] *= (
-                                self.risk_limits.max_position_size / np.abs(
-                            weights[np.abs(weights) > self.risk_limits.max_position_size]))
-
-                # Check sector exposure
-                for sector, indices in sector_groups.items():
-                    sector_sum = np.sum(np.abs(weights[indices]))
-                    if sector_sum > self.risk_limits.max_sector_exposure:
-                        violations = True
-                        scale = self.risk_limits.max_sector_exposure / sector_sum
-                        weights[indices] *= scale
-
-                # Check beta if applicable
-                if betas is not None and len(betas) == n_assets:
-                    beta_exp = abs(np.dot(weights, betas))
-                    if beta_exp > self.risk_limits.max_beta_exposure:
-                        violations = True
-                        weights *= (self.risk_limits.max_beta_exposure / beta_exp)
-
-                if not violations:
-                    break
-            weights = weights / np.sum(np.abs(weights))  # Renormalize
-
-        return weights
-
-
-    def calculate_marginal_var(self, ticker, positions, returns_covariance, confidence=0.05):
-        """Calculate marginal VaR contribution of a position"""
-        if ticker not in positions or returns_covariance is None:
-            return 0.0
-
-        # Calculate VaR with and without the position
-        full_var = self.calculate_portfolio_var(positions, returns_covariance, confidence)
-
-        # Remove position and recalculate
-        positions_without = {k: v for k, v in positions.items() if k != ticker}
-        var_without = self.calculate_portfolio_var(positions_without, returns_covariance, confidence)
-
-        return full_var - var_without
-
-    def check_risk_limits(self, positions, portfolio_value, beta_exposure=0.0):
-        """Check all risk limits and return violations"""
-        violations = []
-
-        if not positions:
-            return violations
-
-        # Position size limits
-        for ticker, pos in positions.items():
-            position_pct = abs(pos['market_value']) / portfolio_value
-            if position_pct > self.risk_limits.max_position_size:
-                violations.append(f"Position size limit violated for {ticker}: {position_pct:.2%}")
-
-        # Beta exposure limit
-        if abs(beta_exposure) > self.risk_limits.max_beta_exposure:
-            violations.append(f"Beta exposure limit violated: {beta_exposure:.3f}")
-
-        # Leverage limit
-        gross_exposure = sum(abs(pos['market_value']) for pos in positions.values())
-        leverage = gross_exposure / portfolio_value
-        if leverage > self.risk_limits.max_leverage:
-            violations.append(f"Leverage limit violated: {leverage:.2f}")
-
-        return violations
-
-    def calculate_risk_adjusted_weights(self, expected_returns, covariance_matrix, risk_aversion=1.0):
-        """Calculate optimal portfolio weights using mean-variance optimization"""
-        if len(expected_returns) == 0 or covariance_matrix is None:
-            return {}
-
-        try:
-            # Regularize covariance matrix
-            regularization = 1e-8 * np.eye(covariance_matrix.shape[0])
-            reg_cov_matrix = covariance_matrix + regularization
-
-            # Solve for optimal weights: w = (1/lambda) * inv(Sigma) * mu
-            inv_cov = linalg.inv(reg_cov_matrix)
-            optimal_weights = np.dot(inv_cov, expected_returns) / risk_aversion
-
-            # Normalize weights
-            total_weight = np.sum(np.abs(optimal_weights))
-            if total_weight > 0:
-                optimal_weights = optimal_weights / total_weight
-
-            return optimal_weights
-
-        except (linalg.LinAlgError, np.linalg.LinAlgError):
-            # Fall back to equal weights if optimization fails
-            n_assets = len(expected_returns)
-            return np.ones(n_assets) / n_assets
-
-
-class PerformanceAnalyzer:
-    """Advanced performance analytics with attribution and benchmarking"""
-
-    def __init__(self, benchmark_returns=None):
-        self.benchmark_returns = benchmark_returns
-        self.performance_history = []
-        self.attribution_history = []
-
-    def calculate_performance_metrics(self, returns, benchmark_returns=None):
-        """Calculate comprehensive performance metrics"""
-        if len(returns) == 0:
-            return {}
-
-        returns_array = np.array(returns)
-        returns_array = returns_array[~np.isnan(returns_array)]
-
-        if len(returns_array) == 0:
-            return {}
-
-        # Basic metrics
-        total_return = np.prod(1 + returns_array) - 1
-        annualized_return = (np.prod(1 + returns_array)) ** (252 / len(returns_array)) - 1
-        annualized_vol = np.std(returns_array) * np.sqrt(252)
-        sharpe_ratio = annualized_return / annualized_vol if annualized_vol > 0 else 0
-
-        # Risk metrics
-        downside_returns = returns_array[returns_array < 0]
-        downside_vol = np.std(downside_returns) * np.sqrt(252) if len(downside_returns) > 0 else 0
-        sortino_ratio = annualized_return / downside_vol if downside_vol > 0 else 0
-
-        # Drawdown metrics
-        cumulative_returns = np.cumprod(1 + returns_array)
-        running_max = np.maximum.accumulate(cumulative_returns)
-        drawdowns = (cumulative_returns - running_max) / running_max
-        max_drawdown = np.min(drawdowns)
-
-        # Skewness and Kurtosis
-        skewness = stats.skew(returns_array)
-        kurtosis = stats.kurtosis(returns_array)
-
-        metrics = {
-            'total_return': total_return,
-            'annualized_return': annualized_return,
-            'annualized_volatility': annualized_vol,
-            'sharpe_ratio': sharpe_ratio,
-            'sortino_ratio': sortino_ratio,
-            'max_drawdown': max_drawdown,
-            'skewness': skewness,
-            'kurtosis': kurtosis,
-            'var_95': np.percentile(returns_array, 5),
-            'cvar_95': np.mean(returns_array[returns_array <= np.percentile(returns_array, 5)])
-        }
-
-        # Benchmark-relative metrics
-        if benchmark_returns is not None and len(benchmark_returns) == len(returns_array):
-            excess_returns = returns_array - np.array(benchmark_returns)
-            tracking_error = np.std(excess_returns) * np.sqrt(252)
-            information_ratio = np.mean(excess_returns) * 252 / tracking_error if tracking_error > 0 else 0
-
-            # Beta calculation
-            covariance = np.cov(returns_array, benchmark_returns)[0, 1]
-            benchmark_variance = np.var(benchmark_returns)
-            beta = covariance / benchmark_variance if benchmark_variance > 0 else 0
-
-            # Alpha calculation
-            risk_free_rate = 0.02 / 252  # Assume 2% annual risk-free rate
-            alpha = (annualized_return - risk_free_rate) - beta * (np.mean(benchmark_returns) * 252 - risk_free_rate)
-
-            metrics.update({
-                'tracking_error': tracking_error,
-                'information_ratio': information_ratio,
-                'beta': beta,
-                'alpha': alpha
-            })
-
-        return metrics
-
-
-class InstitutionalPortfolio:
+class PortfolioManager:
     """Institutional-grade portfolio management system"""
 
     def __init__(self,
@@ -1097,8 +637,9 @@ class InstitutionalPortfolio:
             self.logger.error("CRITICAL: Final leverage validation failed!")
             # Emergency position scaling could be implemented here
 
-        # Update current capital
-        self.current_capital = working_cash
+        # --- NEW: update both cash and portfolio value ---
+        self.current_capital = final_portfolio_state['cash']  # keep cash logic intact
+        self.last_portfolio_value = final_portfolio_state['total_portfolio_value']  # NEW: track true portfolio value
 
         # Log comprehensive summary
         deployment_efficiency = (
@@ -1173,12 +714,17 @@ class InstitutionalPortfolio:
         self.returns.append(daily_return)
         self.historical_returns.append(daily_return)
 
-        # Update drawdown tracking
+        # --- NEW: track last portfolio value for proper drawdown ---
+        self.last_portfolio_value = portfolio_value
+
+        # Update drawdown tracking using total portfolio value
         if portfolio_value > self.peak_value:
             self.peak_value = portfolio_value
         current_drawdown = (self.peak_value - portfolio_value) / self.peak_value
         self.current_drawdown = current_drawdown
 
+
+
         # Enhanced leverage and deployment metrics
         deployed_capital = gross_exposure
         deployment_ratio = deployed_capital / portfolio_value if portfolio_value > 0 else 0
@@ -1187,6 +733,19 @@ class InstitutionalPortfolio:
         leverage_ratio = gross_exposure / portfolio_value if portfolio_value > 0 else 0
 
         # CRITICAL: Leverage compliance check
+
+        # Build dict of market values safely
+        position_values = {}
+        for ticker, pos in self.positions.items():
+            ticker_data = market_data[market_data['ticker'] == ticker]
+            if not ticker_data.empty:
+                close_price = ticker_data.iloc[0]['Close']
+                position_values[ticker] = {'market_value': pos['shares'] * close_price}
+            else:
+                self.logger.warning(f"No market data for {ticker} during leverage check, skipping...")
+                position_values[ticker] = {'market_value': 0.0}
+
+        # Validate leverage
         leverage_compliant = self.validate_no_leverage(
             {ticker: {'market_value': pos['market_value']} for ticker, pos in position_details.items()},
             portfolio_value
@@ -1270,8 +829,10 @@ class InstitutionalPortfolio:
         self.portfolio_values.append(portfolio_value)
         self.performance_log.append(comprehensive_metrics)
 
-        # Apply daily costs
-        self.current_capital -= total_borrow_cost
+        # Apply daily costs ONLY to cash, not total portfolio
+        self.current_capital = cash - total_borrow_cost
+
+
 
         # Log critical compliance violations
         if not leverage_compliant:
