@@ -95,27 +95,38 @@ class Strategy(ABC):
             self.atr_cache[key] = pd.Series(tr).rolling(self.atr_period).mean().to_numpy()
         return self.atr_cache[key]
 
-    def normalize_signals_industry_neutral(self, signals_df, window=60):
+    def normalize_signals_industry_neutral(self, signals_df):
         """
-        Rolling industry-neutral normalization with Numba speed.
-        1. Sector-neutral rolling z-score (per sector, using t-1 info).
-        2. Cross-sectional rolling z-score (per date, using t-1 info).
+        Two-step normalization (faster version):
+        1. Industry-neutral z-score within (date, sector).
+        2. Cross-sectional z-score across all tickers in the same date.
         """
         df = signals_df.copy()
         if 'sector' not in df.columns:
             raise ValueError("Sector column is required for industry-neutral normalization")
 
-        df['signal_descriptor'] = pd.to_numeric(df['signal_descriptor'], errors='coerce')
-
         # --- Step 1: sector-neutral normalization ---
-        df = df.sort_values(['sector', 'date']).reset_index(drop=True)
-        df['signal_descriptor'] = df.groupby('sector', group_keys=False)['signal_descriptor'] \
-            .transform(lambda x: rolling_zscore(x.to_numpy(), window))
+        sector_stats = (
+            df.groupby(['date', 'sector'])['signal_descriptor']
+            .agg(['mean', 'std'])
+            .rename(columns={'mean': 'sector_mean', 'std': 'sector_std'})
+            .reset_index()
+        )
+        df = df.merge(sector_stats, on=['date', 'sector'], how='left')
+        df['signal_descriptor'] = (df['signal_descriptor'] - df['sector_mean']) / df['sector_std']
+        # df['signal_descriptor'] = df['signal_descriptor'].replace([np.inf, -np.inf], 0.0).fillna(0.0)
+        df = df.drop(columns=['sector_mean', 'sector_std'])
 
-        # --- Step 2: cross-sectional normalization (per date) ---
-        df = df.sort_values(['date', 'sector']).reset_index(drop=True)
-        df['signal_descriptor'] = df.groupby('date', group_keys=False)['signal_descriptor'] \
-            .transform(lambda x: rolling_zscore(x.to_numpy(), window))
+        # --- Step 2: cross-sectional normalization ---
+        cross_stats = (
+            df.groupby('date')['signal_descriptor']
+            .agg(['mean', 'std'])
+            .rename(columns={'mean': 'cross_mean', 'std': 'cross_std'})
+            .reset_index()
+        )
+        df = df.merge(cross_stats, on='date', how='left')
+        df['signal_descriptor'] = (df['signal_descriptor'] - df['cross_mean']) / df['cross_std']
+        # df['signal_descriptor'] = df['signal_descriptor'].replace([np.inf, -np.inf], 0.0).fillna(0.0)
+        df = df.drop(columns=['cross_mean', 'cross_std'])
 
         return df
-
